#====================================
#            defaults
#   Copyright 2011 - Nathan Osman
#
#  Keeps a list of default values
#   for configuration settings
#
#   StackApplet is released under
#         the MIT license
#====================================

__default_refresh_rate__ = 300000
__default_theme__        = "dark"
__default_accounts__     = {}
__default_port__         = 8150
__default_language__     = "" # the system default



#===================================
#		   stackapplet
#  Copyright 2011 - Nathan Osman
#
#  Allows StackApplet to receive
#     Win32 notifications
#
#   StackApplet is released under
#		the MIT license
#===================================

# Win32 modules
import win32gui
import win32api
import win32con

stackapplet_instance = None

# Provides the main Py file with a way to
# give us a reference to itself
def register_main_instance(inst):
	
	global stackapplet_instance
	stackapplet_instance = inst

# The window procedure that will receive the
# messages.
def window_procedure(hwnd, msg, wparam, lparam):
	
	if msg == win32con.WM_QUERYENDSESSION:
		
		global stackapplet_instance
		stackapplet_instance.shutdown()
	
	return win32gui.DefWindowProc(hwnd, msg, wparam, lparam)

# Create the window that we
# will use to receive notifications
window_class = win32gui.WNDCLASS()

window_class.lpszClassName = 'test_window'
window_class.lpfnWndProc   = window_procedure

win32gui.RegisterClass(window_class)
hinst = win32api.GetModuleHandle(None)

main_window = win32gui.CreateWindowEx(0,
                                      'test_window',
                                      '',
                                      win32con.WS_POPUP,
                                      0, 0, 0, 0, 0, 0,
                                      hinst,
                                      None)

# and we're done!



#! /usr/bin/python

#===================================
#		   stackapplet
#  Copyright 2011 - Nathan Osman
#
#	 Contains the main class
#	  used by StackApplet
#
#   StackApplet is released under
#		the MIT license
#===================================

# Standard modules
import os
import platform
import signal
import sys
import time
import urllib2
import webbrowser

# The following modules are required
# for supporting translations
import locale
import gettext

# We create a shortcut for the gettext function
# which will later point to the function that can
# be used to retrieve text in the appropriate language.
_ = None

# Global definitions
__application__ = "StackApplet"
__authors__	    = ["Nathan Osman", "Isaiah Heyer"]
__artists__	    = ["Marco Ceppi"]
__translators__ = "translator-credits"
__copyright__   = "Copyright (c) 2011, Nathan Osman"
__license__	    = '''Copyright (c) 2011 Nathan Osman

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.'''
__version__	 = "1.5.1"

# Before we do much else, we are going to
# suppress any warnings - these wreak havok
# on Windows.

if platform.system() == 'Windows':
	import warnings
	warnings.simplefilter('ignore')

# modules required for using GTK
import pygtk
pygtk.require('2.0')
import gtk
import gobject

# Now for our own modules
# =======================

# First try to import the native app
# indicator module. Failing that, use our
# fallback module instead.
try:
	import appindicator
except ImportError:
	import appindicator_replacement as appindicator

# Do the exact same thing with pynotify.
# Use our fallback module if it's not available
try:
	import pynotify
except ImportError:
	import pynotify_replacement as pynotify

# Initialize PyNotify
pynotify.init('StackApplet')

# ... the rest of them.
import config_store    # stores configuration settings / data
import defaults        # list of default settings
import network_thread  # for making anychronous network requests to the API
import preferences     # a web server that allows preferences to be changed

# Messaging menu support requires
try:
	import pymm
	CAN_USE_MM = 1
	
except ImportError:
	CAN_USE_MM = 0

#=======================================
# This is the root class that contains
# all of the core logic for StackApplet
#=======================================
class stackapplet:
	
	# Our constructor
	def __init__ (self):
		
		# Before doing anything, we set the current language
		self.set_language()
		
		# Begin by creating the application indicator which we
		# use for displaying the icon.
		self.indicator = appindicator.Indicator ("stackapplet-desktop-client",
		                                         "stackapplet_grey",
		                                         appindicator.CATEGORY_APPLICATION_STATUS)
		
		self.indicator.set_status(appindicator.STATUS_ACTIVE)
		self.indicator.set_attention_icon("stackapplet")
		
		# Create the settings object so that we
		# can retrieve the list of accounts, preferences,
		# and so on.
		self.prefs = config_store.config_store()
		self.prefs.load_settings()
		
		self.refresh_rate = self.prefs.get_val("refresh_rate", defaults.__default_refresh_rate__)
		language          = self.prefs.get_val("language",     defaults.__default_language__)
		
		if not language == "":
			self.set_language(language)
		
		# Determine which icon set to use.
		if self.prefs.get_val("theme", defaults.__default_theme__) == 'dark':
			icon_to_use = 'stackapplet_grey'
		else:
			icon_to_use = 'stackapplet_light'
		
		self.indicator.set_icon(icon_to_use)
		
		# Create the menus for the accounts
		self.initialize_menu()
		
		# Create the messaging menu entry as well.
		if CAN_USE_MM:
			self.mm = pymm.pymm("/usr/share/applications/stackapplet.desktop")
		
		# Create the threaded network manager
		self.network = network_thread.network_thread(self.process_error)
		
		# Immediately get it fetching the list of sites.
		self.stackauth = None
		self.network.issue_request("http://stackauth.com/1.0/sites?key=" + network_thread.API_KEY,
		                           self.process_stackauth,
		                           None,
		                           True,
		                           1)
		
		# Register one of our member functions with
		# the preference server so that we can supply
		# the requested details as necessary.
		preferences.register_callback(self.preference_callback)
		
		# Finally, begin updating the accounts.
		self.timer = None
		self.update()
	
	# If language == "", then the system default is used
	def set_language(self, language=""):
		
		# Bind the domain
		if not platform.system() == 'Windows':
			gettext.bindtextdomain("stackapplet", "/usr/share/locale")
		
		gettext.textdomain("stackapplet")
		
		if language == None:
			
			# Figure out the system locale
			(lang, enc) = locale.getdefaultlocale()
			
			# Set it to a sensible default if we couldn't
			# figure it out otherwise
			if lang == "":
				lang = "en_US"
		
		else:
			lang = language
		
		# Now create the translation object and
		# register the translation function
		if platform.system() == 'Windows':
			tr = gettext.translation("stackapplet",
									 "po",
									 [lang],
									 fallback=True)
		else:
			tr = gettext.translation("stackapplet",
									 "/usr/share/locale",
									 [lang],
									 fallback=True)
		
		# Now apply tr to our _ shortcut
		global _
		_ = tr.gettext
	
	def initialize_menu(self):
		
		# Create the root GTK menu
		self.root_menu = gtk.Menu()
		
		# Create the Accounts and Tools
		# menu headers
		menu_item_accounts_header = gtk.MenuItem(_("Accounts"))
		menu_item_accounts_header.set_sensitive(False)
		menu_item_accounts_header.show()
		self.root_menu.append(menu_item_accounts_header)
		
		# Now add the accounts to the list.
		# The return value tells us if any accounts
		# were added, and if so, we add a separator
		if self.populate_menu():
			separator = gtk.SeparatorMenuItem()
			separator.show()
			self.root_menu.append(separator)
			
		else:
			menu_item_accounts_header.set_label(_("No Accounts"))
		
		# Now we add the remaining menu items
		menu_item_tools_header = gtk.MenuItem(_("Tools"))
		menu_item_tools_header.set_sensitive(False)
		menu_item_tools_header.show()
		self.root_menu.append(menu_item_tools_header)
		
		# For these items, we can use the stock
		# GTK menuitems
		menu_item_refresh = gtk.ImageMenuItem(stock_id=gtk.STOCK_REFRESH)
		menu_item_prefs   = gtk.ImageMenuItem(stock_id=gtk.STOCK_PREFERENCES)
		menu_item_about   = gtk.ImageMenuItem(stock_id=gtk.STOCK_ABOUT)
		menu_item_quit    = gtk.ImageMenuItem(stock_id=gtk.STOCK_QUIT)
		
		menu_item_refresh.connect("activate", self.menu_refresh_activate)
		menu_item_prefs  .connect("activate", self.menu_prefs_activate)
		menu_item_about  .connect("activate", self.menu_about_activate)
		menu_item_quit   .connect("activate", self.menu_quit_activate)
		
		menu_item_refresh.show()
		menu_item_prefs  .show()
		menu_item_about  .show()
		menu_item_quit   .show()
		
		# Add them all to the menu
		self.root_menu.append(menu_item_refresh)
		self.root_menu.append(menu_item_prefs)
		self.root_menu.append(menu_item_about)
		self.root_menu.append(menu_item_quit)
		
		self.indicator.set_menu(self.root_menu)
	
	def populate_menu(self):
		
		# Reset our accounts container
		self.accounts = []
		
		# Loop through each of the accounts in the list
		entries = self.prefs.get_val("accounts", defaults.__default_accounts__)
		accounts_processed = 0
		
		for entry in entries:
			
			self.create_menu_item(entry)
			accounts_processed += 1
		
		return accounts_processed
	
	def create_menu_item(self, entry):
		
		# Create the menu item
		menu_entry = gtk.MenuItem()
		menu_entry.set_label(_("Loading..."))
		
		# Add it to the menu
		menu_entry.show()
		self.root_menu.insert(menu_entry, 1 + len(self.accounts))
		
		# Create the messaging menu entry
		if CAN_USE_MM:
			mm_source = pymm.pymm_source(entry["site_name"])
			mm_source.set_count(0)
		else:
			mm_source = None
		
		# Now we need to check and see what values
		# are present in the configuration file.
		reputation_on_last_poll = entry['reputation_on_last_poll'] if 'reputation_on_last_poll' in entry else 0
		last_comment_ts         = entry['last_comment_ts']         if 'last_comment_ts'         in entry else 0
		last_answer_ts          = entry['last_answer_ts']          if 'last_answer_ts'          in entry else 0
		unread_reputation       = entry['unread_reputation']       if 'unread_reputation'       in entry else 0
		unread_comments         = entry['unread_comments']         if 'unread_comments'         in entry else 0
		unread_answers          = entry['unread_answers']          if 'unread_answers'          in entry else 0
		notifications           = entry['notifications']           if 'notifications'           in entry else 0
		
		account = { "menu"      : menu_entry,           # the Gtk menu
		            "mm_source" : mm_source,            # the Messaging menu source
		            "site"      : entry['site'],        # the URL of the SE site with this account
		            "site_name" : entry['site_name'],   # the name of the SE site
		            "user_id"   : entry['user_id'],     # the user ID for the site
		            
		            "reputation_on_last_poll"   : reputation_on_last_poll, # the API will never return 0, so this is safe
		            "last_comment_ts"           : last_comment_ts,         # the last timestamp for comments to user
		            "last_answer_ts"	        : last_answer_ts,          # the last timestamp for answers to questions
		            "unread_reputation"         : unread_reputation,       # the amount of rep. earned that hasn't been seen
		            "unread_comments"           : unread_comments,         # the num. of comments that haven't been seen
		            "unread_answers"	        : unread_answers,          # the num. of answers that haven't been seen
		            "notifications"             : notifications }          # whether this item has notifications or not
		
		# If there are any notifications, display
		# the indicator.
		if notifications:
			self.activate_notification(account)
		
		# Set the handlers
		menu_entry.connect("activate", self.menu_item_activate, account)
		
		if CAN_USE_MM:
			mm_source.source.connect("user-display", self.messaging_item_activate, account)
		
		# Add this item to our account list
		self.accounts.append(account)
	
	def process_stackauth(self, data, ignored):
		
		self.stackauth = data
	
	def display_notification(self, title, message, image):
		
		# The image in this case is expected to either
		# be an icon name (in which case we display it)
		# or a complete URL (in which case we fetch it)
		if image.startswith("/"):
			
			# Local path, so display it.
			n = pynotify.Notification(title, message, image)
			n.show()
				
		else:
		
			# Check to see if this image is in the cache
			img_path = os.path.join(os.path.expanduser("~"),".stackapplet")
			img_path = os.path.join(img_path, 'logos')
			img_fn = os.path.join(img_path, image + ".png")
			
			if os.path.isfile(img_fn):
			
				# It's in the cache, so simply display
				# the image.
				n = pynotify.Notification(title, message, img_fn)
				n.show()
			
			else:
			
				# Make sure that the dir exists
				if not os.path.isdir(img_path):
					os.makedirs(img_path)
				
				# We need to determine which site is equivalent to this image.
				for site in self.stackauth['api_sites']:
					
					if site['site_url'] == 'http://' + image + '.com':
						
						self.network.issue_request(site['icon_url'],
						                           self.save_and_display_notification,
						                           { 'title': title,
						                             'message': message,
						                             'image_location': img_fn},
						                           False)
					
	def save_and_display_notification(self, data, info):
	
		# Dump data to the file.
		f = open(info['image_location'], "wb")
		f.write(data)
		f.close()
		
		self.display_notification(info['title'],
		                          info['message'],
		                          info['image_location'])
	
	def activate_notification(self, account):
		
		# Set the color of our indicator
		self.indicator.set_status(appindicator.STATUS_ATTENTION)
		
		# Also set the active state of the messaging menu
		if CAN_USE_MM:
			account['mm_source'].set_notify(True)
	
	# Check to see if there are any pending notifications
	# for any of the accounts.
	def test_notifications(self):
		
		any_notifications = 0
		
		for account in self.accounts:
			
			if account['notifications']:
			
				if CAN_USE_MM:
					account['mm_source'].set_notify(True)
				
				any_notifications = 1
			
			else:
				
				if CAN_USE_MM:
					account['mm_source'].set_notify(False)
		
		if any_notifications:
			self.indicator.set_status(appindicator.STATUS_ATTENTION)
		else:
			self.indicator.set_status(appindicator.STATUS_ACTIVE)
	
	# Process an error that we receive from the
	# network thread
	def process_error(self, data, error_str):
	
		# If it's an account, then at least
		# display a notification in that account
		# that there was an error retrieving the data.
		if data in self.accounts:
			
			data['menu'].set_label(error_str)
	
	# This function is responsible for dispatching
	# the API requests and making sure that when the
	# requests are completed that they are processed
	# by the right functions.
	def update(self):
		
		# Remove the current timer
		if(not self.timer == None):
			gobject.source_remove(self.timer)
		
		for account in self.accounts:
			
			self.update_account(account)
		
		# Now run the update again after the specified
		# timeout has been reached
		self.timer = gobject.timeout_add(self.refresh_rate, self.update)
	
	def update_account(self, account):
		
		# Start by dispatching a request to get the
		# current reputation for this account.
		self.network.issue_api_request(account['site'],
		                               '/users/' + account['user_id'],
		                               self.update_reputation,
		                               account)
		
		self.network.issue_api_request(account['site'],
		                               '/users/' + account['user_id'] + '/mentioned',
		                               self.update_comments,
		                               account,
		                               'fromdate=' + str(account["last_comment_ts"]))
		
		self.network.issue_api_request(account['site'],
		                               '/users/' + account['user_id'] + '/questions',
		                               self.update_answers_1,
		                               account)
	
	def update_reputation(self, data, account):
		
		# First make sure that the account still exists
		if not account in self.accounts:
			return
		
		# First check to see if there is any new reputation
		current_rep = data['users'][0]['reputation']
		
		# Check to see if there has been a change
		if not current_rep == account['reputation_on_last_poll'] and not account['reputation_on_last_poll'] == 0:
			
			# Calculate the change
			change = current_rep - account['reputation_on_last_poll']
			
			# Add this new reputation to the amount of reputation
			# that the user has not seen yet.
			account['unread_reputation'] += change
			
			# Display a notification
			if change > 0:
				message = _("You have gained %s reputation.") % (change)
			else:
				message = _("You have lost %s reputation.") % (abs(change))
			
			self.display_notification(account["site_name"] + ":", message, account['site'])
			
			# Last but not least, we need to set that this item
			# has a pending unread notification
			account['notifications'] = 1
			
			self.activate_notification(account)
			
		# Update the menu captions
		self.set_menu_label(account, current_rep)
			
		# Set this as the new reputation.
		account['reputation_on_last_poll'] = current_rep
	
	def update_comments(self, data, account):
		
		# First make sure that the account still exists
		if not account in self.accounts:
			return
		
		if len(data['comments']) and account["last_comment_ts"]:
			
			# We know there is one or more comments
			
			if len(data['comments']) == 1:
				message = _("%s has posted a comment to you.") % (data['comments'][0]["owner"]["display_name"])
			else:
				message = _("You have received %s new comments.") % (len(data['comments']))
			
			self.display_notification(account["site_name"] + ":", message, account['site'])
			
			account['unread_comments'] += len(data['comments'])
			account['notifications'] = 1
			
			self.set_menu_label(account, account['reputation_on_last_poll'])
			
			self.activate_notification(account)
		
		# Set the timestamp
		account["last_comment_ts"] = int(time.time())
	
	def update_answers_1(self, data, account):
		
		# First make sure that the account still exists
		if not account in self.accounts:
			return
		
		# We are only half of the way there. We have the users'
		# questions... now get the answers to those questions.
		
		# Create a vectorized list of question ids
		id_array = []
		
		for question in data['questions']:
			id_array.append(str(question["question_id"]))
		
		if len(id_array) and account["last_answer_ts"]:
			
			# Join the IDs
			id_str = ";".join(id_array)
			
			# Issue the second part of the request
			self.network.issue_api_request(account['site'],
				                           '/questions/' + id_str + '/answers',
				                           self.update_answers_2,
				                           account,
				                           'fromdate=' + str(account["last_answer_ts"]))
				                           
			# ...and the third part - which will get us a list
			# of comments to the question list we have
			self.network.issue_api_request(account['site'],
				                           '/posts/' + id_str + '/comments',
				                           self.update_post_comments,
				                           account,
				                           'fromdate=' + str(account["last_answer_ts"]))
		
		# Set the timestamp
		account["last_answer_ts"] = int(time.time())
	
	def update_answers_2(self, data, account):
		
		# First make sure that the account still exists
		if not account in self.accounts:
			return
		
		# See if there are any new answers
		if len(data['answers']):
			
			# Display a notification
			message = _("Your have received %s new answer(s) to your questions.") % (len(data['answers']))
			self.display_notification(account["site_name"] + ":", message, account['site'])
			
			account['unread_answers'] += len(data['answers'])
			account['notifications'] = 1
			
			self.set_menu_label(account, account['reputation_on_last_poll'])
			
			self.activate_notification(account)
	
	def update_post_comments(self, data, account):
		
		# See if there are any comments that matched the criterion
		
		# First make sure that the account still exists
		if not account in self.accounts:
			return
		
		# We know there is one or more comments but are they
		# comments that were made by that user?
		for comment in data['comments']:
			if int(account['user_id']) == comment["owner"]["user_id"]:
				data['comments'].remove(comment)
		
		if len(data['comments']):
			
			if len(data['comments']) == 1:
				message = _("%s has posted a comment on your question.") % (data['comments'][0]["owner"]["display_name"])
			else:
				message = _("You have received %s new comments to your question.") % (len(data['comments']))
			
			self.display_notification(account["site_name"] + ":", message, account['site'])
			
			account['unread_comments'] += len(data['comments'])
			account['notifications'] = 1
			
			self.set_menu_label(account, account['reputation_on_last_poll'])
			
			self.activate_notification(account)
		
		# Either way, save the account information
		self.save_account_info()
	
	def set_menu_label(self, account, reputation):
		
		# Construct the label we will be using for the account in the menu
		label = account['site_name'] + ' / ' + str(reputation)
		
		# Check for unread reputation
		if account['unread_reputation']:
			label += _(' [%s reputation]') % (account['unread_reputation'])
		
		# Check for comments
		if account['unread_comments']:
			label += _(' [%s comments]') % (account['unread_comments'])
		
		# Check for new answers
		if account['unread_answers']:
			label += _(' [%s answers]') % (account['unread_answers'])
		
		if account['notifications']:
			label = "* " + label
		
		account['menu'].set_label(label)
		
		# Don't forget about the MM
		if CAN_USE_MM:
			account['mm_source'].set_count(account['unread_comments'] + account['unread_answers'])
	
	def preference_callback(self, command, event, data):
		
		# Process the request that was made
		if command == 'get_settings':
		
			data['accounts']     = self.accounts[:]
			data['theme']        = str(self.prefs.get_val('theme', defaults.__default_theme__))
			data['refresh_rate'] = int(self.refresh_rate)
			data['language']     = str(self.prefs.get_val('language', defaults.__default_language__))
		
		elif command == 'get_translation':
			
			global _
			data['translation'] = _
			
		elif command == 'add_account':
			
			# Add the account
			self.create_menu_item(data)
			
			self.update_account(self.accounts[len(self.accounts) - 1])
			
		elif command == 'remove_account':
			
			index = int(data)
			
			# Remove the menu item and MM entry
			self.root_menu.remove(self.accounts[index]['menu'])
			
			if CAN_USE_MM:
				del self.accounts[index]['mm_source']
			
			# Delete the account
			del self.accounts[index]
			
			self.test_notifications()
		
		elif command == 'reorder':
			
			# We need to grab a copy of the item
			account = self.accounts[int(data['old_index'])]
			
			# Now remove it from the list
			self.accounts.remove(account)
			
			# Insert it into its new location
			self.accounts.insert(int(data['new_index']), account)
			
			# We also need to switch the menu items around too.
			self.root_menu.remove(account['menu'])
			self.root_menu.insert(account['menu'], 1 + int(data['new_index']))
		
		elif command == 'set_rate':
			
			self.refresh_rate = int(data)
			
			self.prefs.set_val('refresh_rate', int(data))
		
		elif command == 'set_theme':
			
			if data == 'dark':
				icon_to_use = 'stackapplet_grey'
			else:
				icon_to_use = 'stackapplet_light'
			
			self.indicator.set_icon(icon_to_use)
			self.prefs.set_val('theme', str(data))
		
		elif command == 'set_language':
			
			self.set_language(str(data))
			self.prefs.set_val('language', str(data))
		
		# Set the event to allow the other thread
		# to continue along.
		event.set()
	
	def menu_item_activate(self, widget, account):
		
		# We need to either reset any unread stuff
		# or if there isn't any, then launch the user's
		# profile page.
		if account['notifications']:
			
			# Reset everything.
			account['notifications']     = 0
			account['unread_reputation'] = 0
			account['unread_comments']   = 0
			account['unread_answers']    = 0
			
			# Reset the menu caption
			self.set_menu_label(account, account['reputation_on_last_poll'])
			
			# Recheck for global notifications
			self.test_notifications()
		
		# Otherwise, display their profile page
		else:
			
			webbrowser.open("http://" + account["site"] + ".com/users/" + str(account["user_id"]))
	
	# Note that I have no clue what the third parameter is since
	# it is not documented ANYWHERE.
	def messaging_item_activate(self, indicator, unknown_data, account):
		
		# Just turn around and call menu_item_activate
		self.menu_item_activate(None, account)
	
	def menu_refresh_activate(self, widget):
		
		self.update()
	
	def menu_prefs_activate(self, widget):
		
		preferences.trigger()
	
	def menu_about_activate(self, widget):
		
		global _,__application__,__author__,__copyright__,__license__,__version__
		
		# Display the about dialog box
		dialog = gtk.AboutDialog()
		
		dialog.set_name(_(__application__))
		dialog.set_authors(__authors__)
		dialog.set_artists(__artists__)
		dialog.set_translator_credits(_(__translators__))
		dialog.set_copyright(_(__copyright__))
		dialog.set_license(_(__license__))
		dialog.set_version(__version__)
		
		if platform.system() == 'Windows':
			dialog.set_logo(gtk.gdk.pixbuf_new_from_file(os.path.join(os.path.dirname(sys.executable), "images/stackapplet.png")))
		else:
			dialog.set_logo_icon_name("stackapplet")
		
		dialog.connect("response",lambda x,y: dialog.destroy())
		
		dialog.run()
	
	def menu_quit_activate(self, widget):
		
		self.shutdown()
		
	# Saves all account information
	def save_account_info(self):
		
		# Save the accounts
		accounts_data = []
		
		for account in self.accounts:
			
			# Take a copy
			copy_of_account = dict(account)
			
			# Remove two items
			del copy_of_account['menu']
			del copy_of_account['mm_source']
			
			accounts_data.append(copy_of_account)
		
		self.prefs.set_val('accounts', accounts_data)
		self.prefs.save_settings()
	
	# This is the safe shutdown function that
	# ensures all accounts' notification data
	# are saved with the accounts' information.
	def shutdown(self):
		
		# We need the network thread to shutdown.
		self.network.request_queue.put([0, None])
		self.network.join()
		
		# ...and we need the preferences thread to
		# shutdown.
		preferences.httpd.shutdown()
		preferences.thread.join()
		
		self.save_account_info();
		
		gtk.main_quit()

stackapplet_instance = None

# Do a quick check for old config files
if os.path.isfile(config_store.CONFIG_FILE_PATH):
	
	dialog = gtk.MessageDialog(None,
		                       gtk.DIALOG_MODAL,
		                       gtk.MESSAGE_INFO,gtk.BUTTONS_OK,
		                       _("StackApplet 1.5.1 has found some old configuration files from a previous version of StackApplet.\n\nThese files will be removed so that StackApplet can continue loading. If you had any accounts registered with StackApplet, you will need to add them again.\n\nNote: StackApplet is no longer a Gnome panel applet. It is an indicator and can be located in your system's notification or tray area."))
	dialog.set_title(_("StackApplet has been upgraded:"))
	dialog.connect("response", lambda x,y: dialog.destroy())
	dialog.run()

	# Delete the old files
	os.remove(config_store.CONFIG_FILE_PATH)

# SIGTERM handler
def signal_handler(a, b):
	
	global stackapplet_instance
	stackapplet_instance.shutdown()

# Create the instance of stackapplet and
# run the main GTK thread
if __name__ == "__main__":

	stackapplet_instance = stackapplet()

	# For Windows, we need our helper module
	if platform.system() == 'Windows':
		import w32_notifications
		w32_notifications.register_main_instance(stackapplet_instance)
	
	signal.signal(signal.SIGTERM, signal_handler)
		
	gtk.main()



#===================================
#		   preferences
#  Copyright 2011 - Nathan Osman
#
#  A web server that allows prefs.
#   to be changed from within the
#         user's browser.
#
#   StackApplet is released under
#		the MIT license
#===================================

# For providing HTML / JS translations
import js_translations
import defaults

import BaseHTTPServer
import os
import platform
import shutil
import sys
import threading
import urlparse
import uuid
import webbrowser

import gobject
gobject.threads_init()

from stackapplet import CAN_USE_MM
import defaults

# Now lets see if we can import the JSON
# module. We try either of these two classes
try:
	import json
except ImportError:
	import simplejson as json

# This is the callback that the main thread would like
# us to use whenever we need to issue a request for data
# from the main thread
main_thread_callback = None

def register_callback(callback):
	
	global main_thread_callback
	main_thread_callback = callback

# Here is a list of files that we can
# serve from the web server
file_list = ['/bg.png',
             '/dark_theme.png',
             '/favicon.ico',
             '/images/ui-bg_diagonals-thick_90_eeeeee_40x40.png',
             '/images/ui-bg_flat_15_cd0a0a_40x100.png',
             '/images/ui-bg_glass_100_e4f1fb_1x400.png',
             '/images/ui-bg_glass_50_3baae3_1x400.png',
             '/images/ui-bg_glass_80_d7ebf9_1x400.png',
             '/images/ui-bg_highlight-hard_100_f2f5f7_1x100.png',
             '/images/ui-bg_highlight-hard_70_000000_1x100.png',
             '/images/ui-bg_highlight-soft_100_deedf7_1x100.png',
             '/images/ui-bg_highlight-soft_25_ffef8f_1x100.png',
             '/images/ui-icons_2694e8_256x240.png',
             '/images/ui-icons_2e83ff_256x240.png',
             '/images/ui-icons_3d80b3_256x240.png',
             '/images/ui-icons_72a7cf_256x240.png',
             '/images/ui-icons_ffffff_256x240.png',
             '/index.html',
             '/jquery.js',
             '/jqueryui.css',
             '/jqueryui.js',
             '/light_theme.png',
             '/main_logo.png',
             '/translations.js']

AUTOSTART_FILE_PATH = os.path.join(os.path.expanduser("~"), ".config/autostart")
AUTOSTART_FILE_NAME = os.path.join(AUTOSTART_FILE_PATH, "stackapplet.desktop")

MM_FILE_PATH = os.path.join(os.path.expanduser("~"), ".config/indicators/messages/applications")
MM_FILE_NAME = os.path.join(MM_FILE_PATH, "stackapplet")

class preference_server(BaseHTTPServer.BaseHTTPRequestHandler):

	nonce      = ''
	auth_token = ''
	
	@staticmethod
	def reset_tokens():
		
		# Generate a nonce and a authorization token
		preference_server.nonce      = uuid.uuid4().hex
		preference_server.auth_token = uuid.uuid4().hex
	
	# Initializes the list of actions available to connecting clients
	def init_actions(self):
		
		# This list is a mapping between request actions
		# and member functions
		self.data_list = [ [ '/get_settings',     self.process_get_settings     ],
		                   [ '/get_translations', self.process_get_translations ],
		                   [ '/add_account',      self.process_add_account      ],
		                   [ '/remove_account',   self.process_remove_account   ],
		                   [ '/reorder',          self.process_reorder          ],
		                   [ '/set_rate',         self.process_set_rate         ],
		                   [ '/set_startup',      self.process_set_startup      ],
		                   [ '/set_mm',           self.process_set_mm           ],
		                   [ '/set_theme',        self.process_set_theme        ],
		                   [ '/set_language',     self.process_set_language     ] ]
	
	def get_action_function(self, action):
	
		for entry in self.data_list:
		
			if action == entry[0]:
				return entry[1]
		
		return None
	
	# Ensures the user is authorized to access the page
	def check_authorization(self):
		
		# Check to see if the user has already received the auth token
		if not self.nonce == '':
			# Get the cookies set by the user
			cookies = self.headers.getheader('Cookie', '').split('; ')
			
			# Check for the stackapplet_auth cookie
			if 'stackapplet_auth=' + self.auth_token in cookies:
				return True
			else:
				self.send_reply(403, "Invalid authorization credentials supplied. Please click on the preferences menu item in the application to view this page.")
				return False
		else:
			self.send_reply(403, "Your session has expired. Please click on the preferences menu item in the application to view this page.")
	
	# There are basically two types of requests
	# that we process - GET requests (for files)
	# and POST requests (for actions).
	def do_GET(self):
		
		# Log the request
		print '[GET]: ' + self.path
		
		# Check to see if the user is authenticating
		if not self.nonce == '' and self.path == '/auth?nonce=' + self.nonce:
			
			# It can only be used once, so drop it
			self.nonce = ''
			
			# Redirect the client to the '/' page
			self.send_response(302)
			
			# Set the auth token as a cookie to the user
			self.send_header('Set-Cookie', 'stackapplet_auth=' + self.auth_token)
			self.send_header('Location', '/')
			
			return
		
		# Make sure the user is authorized
		if not self.check_authorization():
			return
		
		# If no page was specified, default to index.html
		if self.path == '/':
			self.path = '/index.html'
		
		# self.path contains the path that we want
		# to deal with. Check to see if it's one of the
		# pages that we serve static content from.
		global file_list
		
		if self.path in file_list:
			self.process_file()
		else:
			self.send_reply(404, "The page you have requested does not exist on this server.")
	
	def do_POST(self):
		
		# Log the request
		print '[POST]: ' + self.path
		
		# Make sure the user is authorized
		if not self.check_authorization():
			return
		
		self.init_actions()
		
		# Get the POST data
		length = int(self.headers.getheader('content-length'))
		self.post_vars = urlparse.parse_qs(self.rfile.read(length))
		
		function = self.get_action_function(self.path)
		if not function == None:
			function()
		else:
			self.send_reply(404, "The specified command is not recognized by StackApplet.")
	
	def send_reply(self, status, reply, content_type = 'text/plain'):
		
		self.send_response(status)
		self.send_header('Content-length', str(len(reply)))
		self.send_header('Content-type', content_type)
		self.end_headers()
		self.wfile.write(reply)
	
	def process_file(self):
		
		# Now we need to simply construct the path
		# to the file and grab the contents. First check
		# to see if the file is in the html folder.
		filename = os.path.join(sys.path[0], "html" + self.path)
		
		if not os.path.isfile(filename):
			
			if platform.system() == 'Windows':
				filename = os.path.join(os.path.dirname(sys.executable), "html" + self.path)
			else:
				filename = os.path.join(sys.path[0], "../html" + self.path)
		
		f = open(filename, 'rb')
		data = f.read()
		
		if filename.endswith('.png'):
			self.send_reply(200, data, 'image/png')
		elif filename.endswith('.css'):
			self.send_reply(200, data, 'text/css')
		elif filename.endswith('.js'):
			self.send_reply(200, data, 'text/javascript')
		else:
			self.send_reply(200, data, 'text/html')
	
	def process_get_settings(self):
		
		# We need to get the current list of accounts.
		# This means sending a request to the main thread
		# and waiting for a reply with the list of accounts.
		global main_thread_callback
		
		# Now have this message processed in the main
		# thread BUT then wait for it to be signaled
		# once processing is complete
		event = threading.Event()
		data = {}
		json_account = []
		
		gobject.idle_add(main_thread_callback, 'get_settings', event, data)
		event.wait()
		
		# Account list should now have our data
		for account in data['accounts']:
			
			json_account.append({ 'site_name': account['site_name'] })
		
		# Get startup data
		global AUTOSTART_FILE_NAME, MM_FILE_NAME
		
		if os.path.isfile(AUTOSTART_FILE_NAME):
			startup = 1
		else:
			startup = 0
		
		if os.path.isfile(MM_FILE_NAME):
			mm = 1
		else:
			mm = 0
		
		# Determine if we are on Windows
		on_windows = 0
		
		if platform.system() == 'Windows':
			on_windows = 1
		
		# Determine if we have a messaging menu
		can_use_mm = 0
		
		if CAN_USE_MM:
			can_use_mm = 1
		
		response = json.dumps({ 'accounts'    : json_account,
		                        'theme'       : data['theme'],
		                        'refresh_rate': data['refresh_rate'],
		                        'language'    : data['language'],
		                        'startup'     : startup,
		                        'mm'          : mm,
		                        'on_windows'  : on_windows,
		                        'can_use_mm'  : can_use_mm })
		
		self.send_reply(200, response, 'application/json')
	
	def process_get_translations(self):
		
		global main_thread_callback
		data = {}
		
		event = threading.Event()
		gobject.idle_add(main_thread_callback, 'get_translation', event, data)
		event.wait()
		
		# Refresh the translations and return them
		js_translations.refresh(data['translation'])
		json_result = json.dumps(js_translations.translation_keys);
		
		self.send_reply(200, json_result, 'application/json')
	
	def process_add_account(self):
		
		global main_thread_callback
		
		# Get the variables for this account
		site      = self.post_vars['site'][0]
		site_name = self.post_vars['site_name'][0]
		user_id   = self.post_vars['user_id'][0]
		
		event = threading.Event()
		gobject.idle_add(main_thread_callback, 'add_account', event, { 'site'     : site,
		                                                               'site_name': site_name,
		                                                               'user_id'  : user_id })
		event.wait()
		
		self.send_reply(200, '', 'application/json')
	
	def process_remove_account(self):
		
		global main_thread_callback
		
		account_index = self.post_vars['index'][0]
		
		event = threading.Event()
		gobject.idle_add(main_thread_callback, 'remove_account', event, account_index)
		event.wait()
		
		self.send_reply(200, '', 'application/json')
	
	def process_reorder(self):
		
		global main_thread_callback
		
		old_index = self.post_vars['old_index'][0]
		new_index = self.post_vars['new_index'][0]
		
		event = threading.Event()
		gobject.idle_add(main_thread_callback, 'reorder', event, { 'old_index': old_index,
		                                                           'new_index': new_index })
		event.wait()
		
		self.send_reply(200, '', 'application/json')
	
	def process_set_rate(self):
		
		global main_thread_callback
		
		rate = self.post_vars['rate'][0]
		
		event = threading.Event()
		gobject.idle_add(main_thread_callback, 'set_rate', event, rate)
		event.wait()
		
		self.send_reply(200, '', 'application/json')

	def process_set_startup(self):
		
		# We actually don't need the main thread to manipulate
		# this feature.
		global AUTOSTART_FILE_PATH, AUTOSTART_FILE_NAME
		
		# See if the path exists
		if not os.path.isdir(AUTOSTART_FILE_PATH):
			os.mkdir(AUTOSTART_FILE_PATH)
		
		# Now see if we're adding or removing
		value = self.post_vars['value'][0]
		
		if value == '0':
			
			# We're removing
			if os.path.isfile(AUTOSTART_FILE_NAME):
				os.remove(AUTOSTART_FILE_NAME)
		
		else:
		
			# We're copying it there.
			shutil.copyfile('/usr/share/applications/stackapplet.desktop', AUTOSTART_FILE_NAME)
		
		self.send_reply(200, '', 'application/json')
	
	def process_set_mm(self):
		
		# We actually don't need the main thread to manipulate
		# this feature.
		global MM_FILE_PATH, MM_FILE_NAME
		
		# See if the path exists
		if not os.path.isdir(MM_FILE_PATH):
			os.makedirs(MM_FILE_PATH)
		
		# Now see if we're adding or removing
		value = self.post_vars['value'][0]
		
		if value == '0':
			
			# We're removing
			if os.path.isfile(MM_FILE_NAME):
				os.remove(MM_FILE_NAME)
		
		else:
		
			# We're creating the file
			f = open(MM_FILE_NAME, 'w')
			f.write("/usr/share/applications/stackapplet.desktop")
			f.close()
		
		self.send_reply(200, '', 'application/json')
	
	def process_set_theme(self):
		
		global main_thread_callback
		
		theme = self.post_vars['theme'][0]
		
		event = threading.Event()
		gobject.idle_add(main_thread_callback, 'set_theme', event, theme)
		event.wait()
		
		self.send_reply(200, '', 'application/json')
	
	def process_set_language(self):
		
		global main_thread_callback
		
		if 'language' in self.post_vars:
			language = self.post_vars['language'][0]
		else:
			language = ''
		
		event = threading.Event()
		gobject.idle_add(main_thread_callback, 'set_language', event, language)
		event.wait()
		
		self.send_reply(200, '', 'application/json')
	
	# This is here to supplant those annoying
	# status messages that display whenever a client
	# views a page. We provide our own logging facilities ATM.
	def log_message(self, a, *b):
		
		pass

# The port we're listening on
current_port = defaults.__default_port__

# Create the server
while True:
	try:
		httpd = BaseHTTPServer.HTTPServer(('localhost', current_port), preference_server)
		break
	except:
		current_port += 1

# No create a special thread just for the server
thread = threading.Thread(target=httpd.serve_forever)

# Start it
thread.start()

# The handler
def trigger():
	global current_port
	preference_server.reset_tokens()
	webbrowser.open("http://localhost:" + str(current_port) + '/auth?nonce=' + preference_server.nonce)



#=========================
#
#   PyNotify for GTK
#  drop-in replacement
#
#    Copyright 2010
#     Nathan Osman
#
#=========================

import pygtk
import gtk
import gobject

# We also have an init() method that does absolutely nothing
def init(unused_param):
	pass

# We have a queue here that we store the notifications
# in. When someone calls 'notification::show' then the
# notification is either appended to the queue or shown.
notifications_queue = []

class Notification:
	
	def __init__ (self, title, message, icon):
		
		# Start by creating a window that we will use
		# to display the information.
		self.window = gtk.Window(gtk.WINDOW_POPUP)
		
		# Set a few properties on the window,
		# including background color.
		self.window.set_border_width(10)
		self.window.set_opacity(0.8)
		self.window.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse('#000000'))
		
		# Create two labels to insert into the window
		self.title   = gtk.Label()
		self.message = gtk.Label()
		
		self.title.set_markup("<b>" + title + "</b>")
		self.message.set_label(message)
		
		self.title.set_alignment(0,0)
		self.message.set_alignment(0,0)
		self.title.show()
		self.message.show()
		
		# Set some properties on the labels
		self.title.modify_fg(gtk.STATE_NORMAL, gtk.gdk.color_parse('#ffffff'))
		self.message.modify_fg(gtk.STATE_NORMAL, gtk.gdk.color_parse('#ffffff'))
		
		# Create the image
		self.image = gtk.Image()
		pixbuf = gtk.gdk.pixbuf_new_from_file(icon)
		scaled_buf = pixbuf.scale_simple(48, 48, gtk.gdk.INTERP_BILINEAR)
		self.image.set_from_pixbuf(scaled_buf)
		
		self.image.set_size_request(48, 48)
		self.image.set_alignment(0,0)
		self.image.show()
		
		# Now create the layout for the control
		vbox = gtk.VBox()
		vbox.add(self.title)
		vbox.add(self.message)
		vbox.show()
		
		hbox = gtk.HBox()
		hbox.set_spacing(8)
		hbox.add(self.image)
		hbox.add(vbox)
		hbox.show()
		
		self.window.add(hbox)
		
		# Now we need to position the window
		screen = gtk.gdk.Screen()
		rect = screen.get_monitor_geometry(0)
		
		xpos = (rect.x + rect.width) - self.window.get_size()[0] - 32
		ypos = rect.y + 64
		
		self.window.move(xpos, ypos)
		
	def show(self):
		
		global notifications_queue
		
		# If there is nothing in the queue, then
		# do nothing. Otherwise, display this window.
		if not len(notifications_queue):
			
			# Done will be called when the notification is done
			self.display_window()
		
		# Append this item to the queue
		notifications_queue.append(self)
	
	def display_window(self):
		
		self.window.show()
		gobject.timeout_add(10000, self.done)
	
	def done(self):
		
		# Hide this window
		self.window.hide()
		
		global notifications_queue
		
		# Pop the item being displayed
		notifications_queue.remove(self)
		
		# See if there are any others in the queue
		if len(notifications_queue):
			
			# Pop and display that item
			next = notifications_queue[0]
			next.display_window()



#====================================
#          network_thread
#   Copyright 2011 - Nathan Osman
#
#  Manages network reuqests to and
# from the SE API and processes them
#       in a separate thread.
#
#   StackApplet is released under
#         the MIT license
#====================================

import Queue
import threading
import time

import urllib2
import zlib

# We need gobject for cross-thread signalling
import gobject
gobject.threads_init()

# Now lets see if we can import the JSON
# module. We try either of these two classes
try:
	import json
except ImportError:
	import simplejson as json

# Constants
API_VERSION = "1.1"
API_KEY     = "_qlxmEAOH06hLA1_FsZIGQ"

class network_thread(threading.Thread):
	
	def __init__(self, error_handler):
		
		# Initialize the thread object
		threading.Thread.__init__(self)
		
		self.error_handler = error_handler
		
		# The list of items that we will
		# be requesting.
		self.request_queue = Queue.PriorityQueue()
		
		# Start it
		self.start()
	
	def issue_request(self, url, callback, data, decompress=True, priority=2):
		
		# Append the URL to the queue
		self.request_queue.put([priority, [url, callback, data, decompress]])
	
	def issue_api_request(self, site, method, callback, data, additional_params=''):
		
		url = "http://api." + site + ".com/" + API_VERSION + method +"?key=" + API_KEY
		
		if not additional_params == '':
			url += '&' + additional_params
		
		self.issue_request(url, callback, data)
	
	def run(self):
	
		# Now we run in circles waiting
		# for each request to come in.
		while True:
			
			# Get the next item
			item_block = self.request_queue.get(True)
			
			item = item_block[1]
			
			if item == None:
				break;
			
			# Note that we need to ensure that there are at
			# least 100ms between requests, so we simply insert
			# a small sleep here.
			time.sleep(0.1)
			
			try:
			
				# Issue the request
				request = urllib2.Request(item[0])
				request.add_header('Accept-Encoding', 'gzip,deflate')
			
				opener = urllib2.build_opener()
				gzipped_stream = opener.open(request)
			
				# If the request needs decompression, assume
				# that it also needs json decoding.
				if item[3]:
					raw_data = zlib.decompress(gzipped_stream.read(), 16+zlib.MAX_WBITS)
					json_data = json.loads(raw_data)
				else:
					json_data = gzipped_stream.read()			
			
				time.sleep(1)
				gobject.idle_add(item[1], json_data, item[2])
			
			except urllib2.URLError:
				
				# There was an error accessing the URL - a
				# number of different things can cause this.
				gobject.idle_add(self.error_handler, item[2], "Network error.")
			
			except zlib.error:
			
				# There was an error decompressing the stuff.
				gobject.idle_add(self.error_handler, item[2], "GZip error.")
				
			except Exception, e:
				
				gobject.idle_add(self.error_handler, item[2], "Unknown error.")



#=========================
#
# AppIndicator for GTK
#  drop-in replacement
#
#    Copyright 2010
#     Nathan Osman
#
#=========================

# We require PyGTK
import gtk
import gobject

# We also need os and sys
import os
import sys

# Types
CATEGORY_APPLICATION_STATUS = 0

# Status
STATUS_ACTIVE    = 0
STATUS_ATTENTION = 1

# Locations to search for the given icon
search_locations = [ os.path.join(os.path.dirname(sys.executable), "images"),
                     os.path.join(os.path.dirname(sys.path[0]), "images"),
                     os.path.join(os.path.dirname(sys.path[0]), "../images"),
                     '/usr/share/pixmaps' ]

def get_icon_filename(icon_name):
	
	# Determine where the icon is
	global search_locations
	
	for folder in search_locations:
		
		filename = os.path.join(folder, icon_name + ".png")
		
		if os.path.isfile(filename):
			return filename
	
	return None
		

# The main class
class Indicator:
	
	# Constructor
	
	def __init__ (self,unknown,icon,category):
		
		# Store the settings		
		self.inactive_icon = get_icon_filename(icon)
		self.active_icon   = ""  # Blank until the user calls set_attention_icon
		
		# Create the status icon
		self.icon = gtk.StatusIcon()
		
		# Initialize to the default icon
		self.icon.set_from_file(self.inactive_icon)
		
		# Set the rest of the vars
		self.menu = None    # We have no menu yet
	
	def set_menu(self,menu):
		
		# Save a copy of the menu
		self.menu = menu
		
		# Now attach the icon's signal
		# to the menu so that it becomes displayed
		# whenever the user clicks it
		self.icon.connect("activate", self.show_menu)
	
	def set_status(self, status):
		
		# Status defines whether the active or inactive
		# icon should be displayed.
		if status == STATUS_ACTIVE:
			self.icon.set_from_file(self.inactive_icon)
		else:
			self.icon.set_from_file(self.active_icon)
	
	def set_icon(self, icon):
		
		# Set the new icon
		self.icon.set_from_file(get_icon_filename(icon))
	
	def set_attention_icon(self, icon):
		
		# Set the icon filename as the attention icon
		self.active_icon = get_icon_filename(icon)
	
	def show_menu(self, widget):
		
		# Show the menu
		self.menu.popup(None,None,None,0,0)
		
		# Get the location and size of the window
		mouse_rect = self.menu.get_window().get_frame_extents()
		
		self.x = mouse_rect.x
		self.y = mouse_rect.y
		self.right  = self.x + mouse_rect.width
		self.bottom = self.y + mouse_rect.height
		
		# Set a timer to poll the menu
		self.timer = gobject.timeout_add(100, self.check_mouse)
	
	def check_mouse(self):
		
		if not self.menu.get_window().is_visible():
			return
		
		# Now check the global mouse coords
		root = self.menu.get_screen().get_root_window()
		
		x,y,z = root.get_pointer()
		
		if x < self.x or x > self.right or y < self.y or y > self.bottom:
			self.hide_menu()
		else:
			return True
	
	def hide_menu(self):
		
		self.menu.popdown()



#===================================
#		  js_translations
#  Copyright 2011 - Nathan Osman
#
#  Makes it easy for HTML and JS
#       to be translated
#
#   StackApplet is released under
#		the MIT license
#===================================

# This dict will be filled in after the translation information
# is refreshed.
translation_keys = { }

def refresh(_):

	global translation_keys

	# Fill in the translation information
	translation_keys = { 'title': 'StackApplet Preferences',
		                 
		                 'tr_accounts'   : _('Accounts'),
		                 'tr_appearance' : _('Appearance'),
		                 'tr_system'     : _('System'),
		                 
		                 'tr_reorder_hint' : _('<strong>Note:</strong> You can reorder items by dragging them.'),
		                 'tr_account_list' : _('Here are all of the accounts registered with StackApplet:'),
		                 
		                 'tr_loading'          : _('Loading...'),
		                 'tr_account_settings' : _('Account settings:'),
		                 'tr_refresh_rate'     : _('Refresh Rate:'),
		                 
		                 'tr_every_5'  : _('Every 5 minutes'),
		                 'tr_every_8'  : _('Every 8 minutes'),
		                 'tr_every_10' : _('Every 10 minutes'),
		                 'tr_every_15' : _('Every 15 minutes'),
		                 'tr_every_20' : _('Every 20 minutes'),
		                 
		                 'tr_appearance_title' : _('Use the options below to change the appearance of StackApplet:'),
		                 'tr_icon_theme'       : _('Icon theme:'),
		                 'tr_dark'             : _(' Dark'),
		                 'tr_light'            : _(' Light'),
		                 
		                 'tr_behavior' : _('These settings affect the way StackApplet behaves:'),
		                 'tr_startup'  : _(' Load StackApplet when I log in'),
		                 'tr_mm'       : _(' Register StackApplet in the Messaging Menu'),
		                 'tr_language' : _('Language:'),
		                 
		                 'tr_footer' : _('Copyright 2011 - Nathan Osman'),
		                 
		                 'tr_enter_details'      : _('Enter the details for the new account below:'),
		                 'tr_site'               : _('Site:'),
		                 'tr_user_id'            : _('User ID:'),
		                 'tr_find_user'          : _('Find User'),
		                 'tr_find_user_hint'     : _('If you do not know your user ID, click the "find user" button.'),
		                 'tr_check_assoc'        : _('Check for associated accounts'),
		                 'tr_add_account_button' : _('Add Account'),
		                 
		                 'tr_assoc_list'   : _('The following associated accounts were found:'),
		                 'tr_check_all'    : _('check all'),
		                 'tr_uncheck_all'  : _('uncheck all'),
		                 'tr_select_assoc' : _('Please select all of the accounts you would like to add.'),
		                 'tr_add_accounts' : _('Add Accounts'),
		                 'tr_enter_disp'   : _('Please enter part of the user\'s display name and click "search."'),
		                 'tr_search'       : _('Search'),
		                 'tr_searching'    : _('Searching...'),
		                 
		                 'tr_add_account'  : _('add a new account'),
		                 'tr_no_accounts'  : _('No accounts.'),
		                 
		                 'tr_account_dialog_title'    : _('Add New Account'),
		                 'tr_associated_dialog_title' : _('Associated Accounts'),
		                 'tr_find_dialog_title'       : _('Find User:'),
		                 
		                 'tr_success_language' : _('The language was changed successfully but you will need to restart StackApplet in order to apply the new settings to the entire interface.'),
		                 
		                 'tr_error_settings'  : _('There was an error retrieving account settings.'),
		                 'tr_error_remove'    : _('There was an error removing the account.'),
		                 'tr_error_add'       : _('There was an error adding the new account.'),
		                 'tr_error_lookup'    : _('There was an error looking up your user ID.'),
		                 'tr_error_check'     : _('There was an error checking for associated accounts.'),
		                 'tr_error_swap'      : _('There was an error swapping the two items.'),
		                 'tr_error_theme'     : _('There was an error changing the theme.'),
		                 'tr_error_refresh'   : _('There was an error changing the refresh rate.'),
		                 'tr_error_language'  : _('There was an error setting the current language.'),
		                 'tr_error_add_assoc' : _('There was an error adding an account.'),
		                 'tr_error_startup'   : _('There was an error setting this application to load at startup.'),
		                 'tr_error_search'    : _('There was an error searching for users.'),
		                 'tr_error_stackauth' : _('There was an error loading the list of sites from StackAuth.')
	}



#===================================
#         config_store
#  Copyright 2010 - Nathan Osman
#
#   Provides a convenient way to
#   store settings pertaining to
# the preferences of the application
#
#   StackApplet is released under
#        the MIT license
#===================================

import os

CONFIG_FILE_PATH = os.path.join(os.path.expanduser("~"),".stackapplet")
CONFIG_FILE_NAME = os.path.join(CONFIG_FILE_PATH,"config")

# We need to import a JSON library
# Here we try to load two common ones

try:
    import json
except ImportError:
    import simplejson as json

class config_store:

	# Constructor
	
	def __init__(self):
		
		# We'll start with a blank slate
		
		self.data = {}
	
	# This function loads the settings from
	# our config file
	def load_settings(self):
		
		# Begin by trying to open the file
		
		try:
		
			global CONFIG_FILE_PATH, CONFIG_FILE_NAME
			
			f = open(CONFIG_FILE_NAME,'r')
			
			# The file is open... read the JSON
			# data from it
			
			self.data = json.loads(f.readline())
			
			f.close()
			
		except Exception,e:
		
			# If there is a problem (like the file
			# can't be opened) just ignore it
			
			pass
	
	def save_settings(self):
		
		# Begin by serializing the contents of our
		# data collection
		
		serialized_data = json.dumps(self.data)
		
		# Make sure that the directories that we need exist
		
		try:
			global CONFIG_FILE_PATH
			
			os.makedirs(CONFIG_FILE_PATH)
			
		except Exception,e:
		
			# If there is a problem, assume that
			# the dorectory already exists
			
			pass
		
		f = open(CONFIG_FILE_NAME,'w')
		
		f.write(json.dumps(self.data))
		
		f.close()
	
	def get_val(self,name,default_value):
		
		# Check to see if the key
		# exists in our data collection
		
		if name in self.data:
			return self.data[name]
		else:
			# We also want to store the default
			# value so that next time, we'll get
			# that instead
			
			self.data[name] = default_value
			return default_value;
	
	def set_val(self,name,value):
		
		self.data[name] = value



#===================================
#              pymm
#  Copyright 2010 - Nathan Osman
#
#     A small wrapper around
#       the messaging menu
#
#   StackApplet is released under
#        the MIT license
#===================================

# Import the indicate module
import indicate

class pymm:
    
    def __init__(self, desktop_file):
        
        # Create the server
        self.server = indicate.indicate_server_ref_default()
        
        # Set the server's desktop file
        self.server.set_desktop_file(desktop_file)
        
        # Set the type
        self.server.set_type("message.im")
        
        # Now show the item
        self.server.show()

class pymm_source:

    def __init__(self, name, icon = None):
        
        self.source = indicate.Indicator()
        
        self.source.set_property("subtype", "im")
        self.source.set_property("sender", name)
        
        if icon:
            self.source.set_property("icon", icon)
        
        self.source.show()
        
    def set_count(self, count):
    
        self.source.set_property("count", str(count))
    
    def set_notify(self, notify):
    	
    	if notify:
    		self.source.set_property("draw-attention", "true")
    	else:
    		self.source.set_property("draw-attention", "false")
